@@ -6,13 +6,24 @@ df2lambda <- function(X, df = 4, lambda = NULL, dmat = NULL, weights,
 
 
     stopifnot(xor(is.null(df), is.null(lambda)))
-    if (!is.null(df))
-        if (df >= ncol(X)) return(c(df = df, lambda = 0))
+    if (!is.null(df)) {
+        rank_X <- rankMatrix(X, method = 'qr')
+        if (df >= rank_X) {
+            if (df > rank_X)
+                warning(sQuote("df"),
+                        " too large:\n  Degrees of freedom cannot be larger",
+                        " than the rank of the design matrix.\n",
+                        "  Unpenalized base-learner with df = ",
+                        rank_X, " used. Re-consider model specification.")
+            return(c(df = df, lambda = 0))
+        }
+    }
     if (!is.null(lambda))
-        if (lambda == 0) return(c(df = ncol(X), lambda = 0))
+        if (lambda == 0)
+            return(c(df = rankMatrix(X), lambda = 0))
 
     ## check for possible instability
-    if (max(abs(X)) > 10)
+    if (options("mboost_check_df2lambda")[[1]] && max(abs(X)) > 10)
         warning("Some absolute values in design matrix are greater 10. Hence, ",
                 sQuote("df2lambda"), " might be numerically instable.\n  ",
                 "See documentation of argument ", sQuote("by"),
@@ -32,9 +43,15 @@ df2lambda <- function(X, df = 4, lambda = NULL, dmat = NULL, weights,
         dmat <- diag(ncol(XtX))
     }
     A <- XtX + dmat * options("mboost_eps")[[1]]
+    ## make sure that A is also numerically symmetric
+    if (is(A, "Matrix"))
+        A <- forceSymmetric(A)
     Rm <- solve(chol(A))
-    decomp <- svd(crossprod(Rm, dmat) %*% Rm)
-    d <- decomp$d
+    ## singular value decomposition without singular vectors
+    d <- try(svd(crossprod(Rm, dmat) %*% Rm, nu=0, nv=0)$d)
+    ## if unsucessfull try the same computation but compute singular vectors as well
+    if (inherits(d, "try-error"))
+        d <- svd(crossprod(Rm, dmat) %*% Rm)$d
 
     ### option
     if (options("mboost_dftraceS")[[1]]){
@@ -200,9 +217,10 @@ hyper_bbs <- function(mf, vary, knots = 20, boundary.knots = NULL, degree = 3,
     if (cyclic & constraint != "none")
         stop("constraints not implemented for cyclic B-splines")
     stopifnot(is.numeric(deriv) & length(deriv) == 1)
+    ## prediction is usually set in/by newX()
     list(knots = ret, degree = degree, differences = differences,
          df = df, lambda = lambda, center = center, cyclic = cyclic,
-         Ts_constraint = constraint, deriv = deriv)
+         Ts_constraint = constraint, deriv = deriv, prediction = FALSE)
 }
 
 ### model.matrix for P-splines baselearner (including tensor product P-splines)
@@ -210,13 +228,14 @@ X_bbs <- function(mf, vary, args) {
 
     stopifnot(is.data.frame(mf))
     mm <- lapply(which(colnames(mf) != vary), function(i) {
-        X <- bsplines(mf[[i]],
-                      knots = args$knots[[i]]$knots,
-                      boundary.knots = args$knots[[i]]$boundary.knots,
-                      degree = args$degree,
-                      Ts_constraint = args$Ts_constraint,
-                      deriv = args$deriv)
-        if (args$cyclic) {
+        if (!args$cyclic) {
+            X <- bsplines(mf[[i]],
+                          knots = args$knots[[i]]$knots,
+                          boundary.knots = args$knots[[i]]$boundary.knots,
+                          degree = args$degree,
+                          Ts_constraint = args$Ts_constraint,
+                          deriv = args$deriv, extrapolation = args$prediction)
+        } else { ## if cyclic spline
             X <- cbs(mf[[i]],
                      knots = args$knots[[i]]$knots,
                      boundary.knots = args$knots[[i]]$boundary.knots,
@@ -559,8 +578,14 @@ bbs <- function(..., by = NULL, index = NULL, knots = 20, boundary.knots = NULL,
 }
 
 ### cyclic B-splines
-### adapted version of mgcv:cSplineDes from S.N. Wood
+### adapted version of mgcv::cSplineDes from S.N. Wood
 cbs <- function (x, knots, boundary.knots, degree = 3, deriv = 0L) {
+
+    if (any(x < boundary.knots[1], na.rm = TRUE) |
+        any(x > boundary.knots[2], na.rm = TRUE))
+        stop("some ", sQuote("x"), " values are beyond ",
+             sQuote("boundary.knots"))
+
     nx <- names(x)
     x <- as.vector(x)
     ## handling of NAs
@@ -593,14 +618,30 @@ cbs <- function (x, knots, boundary.knots, degree = 3, deriv = 0L) {
     attr(X, "degree") <- degree
     attr(X,"knots") <- knots
     attr(X,"boundary.knots") <- boundary.knots
-    if (deriv != 0)
+    if (length(deriv) > 1 || deriv != 0)
         attr(X, "deriv") <- deriv
     dimnames(X) <- list(nx, 1L:ncol(X))
     return(X)
 }
 
 bsplines <- function(x, knots, boundary.knots, degree,
-                     Ts_constraint = "none", deriv = 0L){
+                     Ts_constraint = "none", deriv = 0L,
+                     extrapolation = FALSE) {
+
+    ## do not allow data beyond boundary knots while fitting
+    if (!extrapolation && (any(x < boundary.knots[1], na.rm = TRUE) |
+                               any(x > boundary.knots[2], na.rm = TRUE)))
+        stop("some ", sQuote("x"), " values are beyond ",
+             sQuote("boundary.knots"))
+
+    ## allow extrapolation when predicting
+    if (extrapolation <- extrapolation &&
+        (any(x < boundary.knots[1], na.rm = TRUE) |
+             any(x > boundary.knots[2], na.rm = TRUE))) {
+        warning("Some ", sQuote("x"), " values are beyond ",
+                sQuote("boundary.knots"), "; Linear extrapolation used.")
+    }
+
     nx <- names(x)
     x <- as.vector(x)
     ## handling of NAs
@@ -618,6 +659,27 @@ bsplines <- function(x, knots, boundary.knots, degree,
     ## construct design matrix
     X <- splineDesign(k, x, degree + 1, derivs = rep(deriv, length(x)),
                       outer.ok = TRUE)
+
+    ## code along the lines of mgcv::Predict.matrix.pspline.smooth
+    if (extrapolation) {
+        ## Build matrix to map coeficients to value (deriv = 0) and
+        ## slope (deriv = 1) at end points.
+        if (deriv != 0L) {
+            warning("deriv != 0L; Linear extrapolation overwritten")
+        } else {
+              deriv <- c(0, 1, 0, 1)
+          }
+        D <- splineDesign(knots = k, x = rep(boundary.knots, each = 2),
+                          ord = degree + 1, deriv)
+        ## Add rows for linear extrapolation
+        idx <- x < boundary.knots[1]
+        if (any(idx, na.rm = TRUE))
+            X[idx,] <- cbind(1, x[idx] - boundary.knots[1]) %*% D[1:2, ]
+        idx <- x > boundary.knots[2]
+        if (any(idx, na.rm = TRUE))
+            X[idx,] <- cbind(1, x[idx] - boundary.knots[2]) %*% D[3:4, ]
+    }
+
     ## handling of NAs
     if (nas) {
         tmp <- matrix(NA, length(nax), ncol(X))
@@ -638,7 +700,7 @@ bsplines <- function(x, knots, boundary.knots, degree,
         attr(X, "Ts_constraint") <- Ts_constraint
     if (Ts_constraint != "none")
         attr(X, "D") <- D
-    if (deriv != 0)
+    if (length(deriv) > 1 || deriv != 0)
         attr(X, "deriv") <- deriv
     dimnames(X) <- list(nx, 1L:ncol(X))
     return(X)
@@ -651,15 +713,12 @@ bl_lin <- function(blg, Xfun, args) {
     index <- blg$get_index()
     vary <- blg$get_vary()
 
-    newX <- function(newdata = NULL) {
+    newX <- function(newdata = NULL, prediction = FALSE) {
         if (!is.null(newdata)) {
-            stopifnot(all(names(blg) %in% names(newdata)))
-            stopifnot(all(class(newdata) == class(mf)))
-            nm <- names(blg)
-            if (any(duplicated(nm)))  ## removes duplicates
-                nm <- unique(nm)
-            mf <- newdata[, nm, drop = FALSE]
+            mf <- check_newdata(newdata, blg, mf)
         }
+        ## this argument is currently only used in X_bbs --> bsplines
+        args$prediction <- prediction
         return(Xfun(mf, vary, args))
     }
     X <- newX()
@@ -728,7 +787,7 @@ bl_lin <- function(blg, Xfun, args) {
         hatvalues <- function() {
             ret <- as.matrix(tcrossprod(X %*% solve(XtX), X * w))
             if (is.null(index)) return(ret)
-            return(ret[index,index])
+            return(ret[index, index])
         }
         ### </FIXME>
 
@@ -738,20 +797,18 @@ bl_lin <- function(blg, Xfun, args) {
         ### prepare for computing predictions
         predict <- function(bm, newdata = NULL, aggregate = c("sum", "cumsum", "none")) {
             cf <- sapply(bm, coef)
-            if (!is.matrix(cf)) cf <- matrix(cf, nrow = 1)
+            if (!is.matrix(cf))
+                cf <- matrix(cf, nrow = 1)
             if(!is.null(newdata)) {
                 index <- NULL
-                nm <- names(blg)
-                if (any(duplicated(nm)))  ## removes duplicates
-                    nm <- unique(nm)
-                newdata <- newdata[,nm, drop = FALSE]
-                ### option
-                if (nrow(newdata) > options("mboost_indexmin")[[1]]) {
+                ## Use sparse data represenation if data set is huge
+                ## and a data.frame
+                if (is.data.frame(newdata) && nrow(newdata) > options("mboost_indexmin")[[1]]) {
                     index <- get_index(newdata)
-                    newdata <- newdata[index[[1]],,drop = FALSE]
+                    newdata <- newdata[index[[1]], , drop = FALSE]
                     index <- index[[2]]
                 }
-                X <- newX(newdata)$X
+                X <- newX(newdata, prediction = TRUE)$X
             }
             aggregate <- match.arg(aggregate)
             pr <- switch(aggregate, "sum" =
@@ -761,8 +818,9 @@ bl_lin <- function(blg, Xfun, args) {
                                PACKAGE = "mboost"), "matrix")
             },
             "none" = as(X %*% cf, "matrix"))
-            if (is.null(index)) return(pr[,,drop = FALSE])
-            return(pr[index,,drop = FALSE])
+            if (is.null(index))
+                return(pr[ , , drop = FALSE])
+            return(pr[index, ,drop = FALSE])
         }
 
         ret <- list(fit = fit, hatvalues = hatvalues,
